@@ -18,6 +18,37 @@ def castable(obj, cast: Any) -> bool:
         return is_castable
 
 
+def load_json(filepath: Union[str, Path]) -> Union[Dict, List, None]:
+    ret = None
+    try:
+        ext = os.path.splitext(filepath)[1].strip()
+        if ext == "" or ext == ".json":
+            ret = json.load(open(filepath, "r", encoding="utf-8"))
+        elif ext == ".jsonl":
+            ret = [
+                json.load(open(line.strip(), "r", encoding="utf-8"))
+                for line in open(filepath, "r", encoding="utf-8").readlines()
+            ]
+    except json.JSONDecodeError:
+        pass
+    finally:
+        return ret
+
+
+def load_txt(filepath: Union[str, Path]) -> Union[Dict, List, None]:
+    ret = [
+        line.strip()
+        for line in open(filepath, "r", encoding="utf-8").readlines()
+    ]
+    return ret
+
+
+def load_file(filepath: Union[str, Path]) -> Union[Dict, List, None]:
+    content = load_json(filepath)
+    content = content or load_txt(filepath)
+    return content
+
+
 def produce_taxonomy(
     documents: Union[str, List[Dict]],
     output_dir: str,
@@ -31,10 +62,7 @@ def produce_taxonomy(
         subcat_cat_rel = 1
 
     if isinstance(documents, str):
-        if documents.endswith(".jsonl"):
-            documents = [json.loads(js) for js in open(documents).readlines()]
-        else:
-            documents = json.load(open(documents))
+        documents = load_file(documents)
 
     doc0 = documents[0]
     gen_ids = "category_id" not in doc0 or "subcategory_id" not in doc0
@@ -285,10 +313,11 @@ class LabelIndex(object):
                     filepath = fpath
                     break
 
-        if str(filepath).endswith(".txt"):
-            index = [line.strip() for line in open(filepath).readlines()]
-        else:
-            index = json.load(open(filepath))
+        index = load_file(filepath)
+        if index is None:
+            raise RuntimeError(
+                f"Unable to load index from {filepath}"
+            )
 
         if isinstance(index, list):
             index = {value: str(idx) for idx, value in enumerate(index)}
@@ -454,15 +483,21 @@ class Taxonomy(object):
         import sys
         import importlib
         from datasets import DownloadManager
-        data_path = str(data_path).strip()
-        if data_path.endswith("/"):
-            data_path = data_path[:-1]
+        # can be local path or pip-package (:
+        if Path(data_path).is_dir():
+            data_path = str(data_path).strip()
+            if data_path.endswith("/"):
+                data_path = data_path[:-1]
 
-        sys.path.append(data_path)
-        data_module = importlib.import_module(basename(data_path))
-        urls = getattr(data_module, "_URLS")
+            sys.path.append(data_path)
+            module_name = basename(data_path)
+        else:
+            module_name = data_path
+
+        data_module = importlib.import_module(module_name)
+        downloader = getattr(data_module, "custom_download")
         dlm = DownloadManager()
-        downloaded = dlm.download_and_extract(urls)
+        downloaded = downloader(dlm)
         return Taxonomy(
             category_index_file=downloaded["cat_labels"],
             category_conversion_file=downloaded["cat2subcat"],
